"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.dependencies = dependencies;
exports.buildD3Data = buildD3Data;
exports.generateHTMLDir = generateHTMLDir;

var _fs = _interopRequireDefault(require("fs"));

var _path = _interopRequireDefault(require("path"));

var _constant = require("../constant/constant");

var _bundle = require("../core/bundle");

var _dependencyGraph = require("../core/dependency-graph");

var _helpers = require("../utils/helpers");

var _repository = require("../core/repository");

var _config = require("../utils/config");

var _i18n = _interopRequireDefault(require("../i18n"));

var _log = _interopRequireDefault(require("../core/log"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _createForOfIteratorHelper(o, allowArrayLike) { var it; if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = o[Symbol.iterator](); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it["return"] != null) it["return"](); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function dependencies() {
  return _dependencies.apply(this, arguments);
}

function _dependencies() {
  _dependencies = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
    var cwd, bundle, repo, data;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.prev = 0;
            cwd = process.cwd();
            bundle = _bundle.Bundle.from(cwd);
            repo = new _repository.Repository(_path["default"].join(cwd, _constant.DEP_FOLDER_NAME), (0, _config.getGlobalRepo)());
            _context.next = 6;
            return buildD3Data(bundle, repo);

          case 6:
            data = _context.sent;
            generateHTMLDir(data, cwd);
            _context.next = 15;
            break;

          case 10:
            _context.prev = 10;
            _context.t0 = _context["catch"](0);
            process.exitCode = -1;

            _i18n["default"].log("dependency.error", {
              error: (0, _helpers.getErrorMessage)(_context.t0)
            });

            _log["default"].debug(_context.t0);

          case 15:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, null, [[0, 10]]);
  }));
  return _dependencies.apply(this, arguments);
}

function buildD3Data(_x, _x2) {
  return _buildD3Data.apply(this, arguments);
}

function _buildD3Data() {
  _buildD3Data = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(bundle, repo) {
    var manifestGraph, nodes, links, buildDataTree;
    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            buildDataTree = function _buildDataTree(list, parentManifest, optionalDeps) {
              var _iterator = _createForOfIteratorHelper(list),
                  _step;

              try {
                var _loop = function _loop() {
                  var manifest = _step.value;
                  var subList = manifestGraph.dependencies(manifest);

                  if (!nodes.find(function (n) {
                    return n.name === manifest.name;
                  })) {
                    nodes.push({
                      name: manifest.name,
                      version: manifest.version.version,
                      publishAs: manifest.publishAs,
                      isOptional: optionalDeps.includes(manifest.name)
                    });
                  }

                  if (parentManifest) {
                    links.push({
                      source: parentManifest.name,
                      target: manifest.name,
                      value: 1
                    });
                  }

                  var subOptionalDeps = manifest.dependencies.filter(function (d) {
                    return d.isOptional;
                  }).map(function (d) {
                    return d.name;
                  });
                  buildDataTree(subList, manifest, subOptionalDeps);
                };

                for (_iterator.s(); !(_step = _iterator.n()).done;) {
                  _loop();
                }
              } catch (err) {
                _iterator.e(err);
              } finally {
                _iterator.f();
              }
            };

            _context2.next = 3;
            return _dependencyGraph.DependencyGraph.build(bundle, repo);

          case 3:
            manifestGraph = _context2.sent;
            nodes = [];
            links = [];
            buildDataTree([bundle.manifest], null, []);
            return _context2.abrupt("return", {
              nodes: nodes,
              links: links
            });

          case 8:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));
  return _buildD3Data.apply(this, arguments);
}

function generateHTMLDir(data, dirPath) {
  var folder = _path["default"].join(dirPath, _constant.DEPS_VISUAL);

  if (!_fs["default"].existsSync(folder)) {
    (0, _helpers.runShellCmd)(function (shell) {
      return shell.mkdir('-p', folder);
    });
  }

  var htmlPath = _path["default"].join(dirPath, _constant.DEPS_VISUAL, 'deps.html');

  _fs["default"].writeFileSync(htmlPath, getHTMLTemplate());

  var jsPath = _path["default"].join(dirPath, _constant.DEPS_VISUAL, 'deps-data.js');

  _fs["default"].writeFileSync(jsPath, "const data = ".concat(JSON.stringify(data)));
}

function getHTMLTemplate() {
  return "\n<html>\n\n<head>\n    <script src=\"./deps-data.js\"></script>\n    <script src=\"https://d3js.org/d3.v5.js\"></script>\n    <script src=\"https://unpkg.com/d3-sankey@0\"></script>\n    <style>\n        .optional {\n            opacity: 0.4;\n        }\n    </style>\n</head>\n\n<body>\n    <div id=\"root\"></div>\n\n    <script type=\"module\">\n        import { Library } from \"https://cdn.jsdelivr.net/npm/@observablehq/runtime@4/dist/runtime.js\";\n\n        const { DOM } = new Library();\n\n        const width = 1366;\n        const height = 500;\n\n        const sankey = (data) => {\n            const sankey = d3.sankey()\n                .nodeId(d => d.name)\n                .nodeAlign(d3[\"sankeyLeft\"])\n                .nodeWidth(15)\n                .nodePadding(10)\n                .extent([[1, 5], [width - 1, height - 5]]);\n            const { nodes, links } = data;\n            return sankey({\n                nodes: nodes.map(d => Object.assign({}, d)),\n                links: links.map(d => Object.assign({}, d))\n            });\n        }\n\n        const genColor = d3.scaleOrdinal(d3.schemeCategory10);\n\n        const color = (d) => {\n            const publishAs = d.publishAs;\n            let color = \"#999\";\n            if (publishAs === \"source\") {\n                color = \"#51A8F9\";\n            } else if (publishAs === \"binary\") {\n                color = \"#71BF40\";\n            } else if (publishAs === \"distribution\") {\n                color = \"#EC5D57\";\n            } else if (publishAs === \"code-segment\") {\n                color = \"#F49013\";\n            }\n            return color;\n        }\n\n        const svg = d3.create(\"svg\")\n            .attr(\"viewBox\", [0, 0, width, height]);\n\n        const { nodes, links } = sankey(data);\n\n        svg.append(\"g\")\n            .attr(\"stroke\", \"#000\")\n            .selectAll(\"rect\")\n            .data(nodes)\n            .join(\"rect\")\n            .attr(\"x\", d => d.x0)\n            .attr(\"y\", d => d.y0)\n            .attr(\"height\", d => d.y1 - d.y0)\n            .attr(\"width\", d => d.x1 - d.x0)\n            .attr(\"fill\", (d) => color(d))\n            .attr(\"class\", d => d.isOptional ? \"optional\" : \"\")\n            .append(\"title\")\n            .text(d => \"publishAs: \" + d.publishAs + \"\\n\" + \"isOptional: \" + d.isOptional);\n\n        const link = svg.append(\"g\")\n            .attr(\"fill\", \"none\")\n            .attr(\"stroke-opacity\", 0.5)\n            .selectAll(\"g\")\n            .data(links)\n            .join(\"g\")\n            .style(\"mix-blend-mode\", \"multiply\");\n\n        const edgeColor = \"path\";\n\n        if (edgeColor === \"path\") {\n            const gradient = link.append(\"linearGradient\")\n                .attr(\"id\", d => (d.uid = DOM.uid(\"link\")).id)\n                .attr(\"gradientUnits\", \"userSpaceOnUse\")\n                .attr(\"x1\", d => d.source.x1)\n                .attr(\"x2\", d => d.target.x0);\n\n            gradient.append(\"stop\")\n                .attr(\"offset\", \"0%\")\n                .attr(\"stop-color\", d => color(d.source));\n\n            gradient.append(\"stop\")\n                .attr(\"offset\", \"100%\")\n                .attr(\"stop-color\", d => color(d.target));\n        }\n\n        link.append(\"path\")\n            .attr(\"d\", d3.sankeyLinkHorizontal())\n            .attr(\"stroke\", d => edgeColor === \"none\" ? \"#aaa\"\n                : edgeColor === \"path\" ? d.uid\n                    : edgeColor === \"input\" ? color(d.source)\n                        : color(d.target))\n            .attr(\"stroke-width\", d => Math.max(1, d.width));\n\n        link.append(\"title\")\n            .text(d => d.source.name + \" \u2192 \" + d.target.name);\n\n        svg.append(\"g\")\n            .attr(\"font-family\", \"sans-serif\")\n            .attr(\"font-size\", 10)\n            .selectAll(\"text\")\n            .data(nodes)\n            .join(\"text\")\n            .attr(\"x\", d => d.x0 < width / 2 ? d.x1 + 6 : d.x0 - 6)\n            .attr(\"y\", d => (d.y1 + d.y0) / 2)\n            .attr(\"dy\", \"0.35em\")\n            .attr(\"text-anchor\", d => d.x0 < width / 2 ? \"start\" : \"end\")\n            .text(d => d.name + \" (\" + d.version + \")\");\n\n        const svgNode = svg.node();\n\n        const rootDom = document.getElementById(\"root\");\n        rootDom.appendChild(svgNode);\n    </script>\n</body>\n\n</html>\n    ";
}