"use strict";

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.install = install;
exports.installBundle = installBundle;
exports.parseInstallParam = parseInstallParam;
exports.traverseDownload = traverseDownload;

var _fs = _interopRequireDefault(require("fs"));

var _i18n = _interopRequireWildcard(require("../i18n"));

var _bundle = require("../core/bundle");

var _repository = require("../core/repository");

var _path = _interopRequireDefault(require("path"));

var _semver = _interopRequireDefault(require("semver"));

var _constant = require("../constant/constant");

var _manifest = require("../core/manifest");

var _dependency = require("../core/dependency");

var _tools = require("../thread/tools");

var _config = require("../utils/config");

var _log = _interopRequireDefault(require("../core/log"));

var _helpers = require("../utils/helpers");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

var _require = require('worker_threads'),
    Worker = _require.Worker;

function install(_x) {
  return _install.apply(this, arguments);
}

function _install() {
  _install = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(bundleDescriptor) {
    var options,
        repo,
        installPath,
        bundle,
        _args = arguments;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            options = _args.length > 1 && _args[1] !== undefined ? _args[1] : {};
            _context.prev = 1;
            repo = null;

            if (!((options.global || options.saveDev) && !bundleDescriptor)) {
              _context.next = 5;
              break;
            }

            throw (0, _i18n.getI18nMessage)("install.nameIsRequired");

          case 5:
            if (options.global) {
              repo = new _repository.Repository('', (0, _config.getGlobalRepo)());
            } else {
              repo = new _repository.Repository(_path["default"].join(process.cwd(), _constant.DEP_FOLDER_NAME), (0, _config.getGlobalRepo)());
            }

            installPath = (0, _config.getRootBundlePath)(options.global);
            bundle = new _bundle.Bundle.from(installPath);
            (0, _repository.createDirIfNotExists)(_constant.DEP_FOLDER_NAME, installPath);
            _context.next = 11;
            return installBundle(bundle, repo, bundleDescriptor, {
              isGlobal: options.global,
              type: options.saveDev ? 'dev' : ''
            });

          case 11:
            _i18n["default"].log('install.success');

            _context.next = 19;
            break;

          case 14:
            _context.prev = 14;
            _context.t0 = _context["catch"](1);

            _i18n["default"].log('install.error', {
              error: (0, _helpers.getErrorMessage)(_context.t0)
            });

            _log["default"].debug(_context.t0);

            process.exitCode = -1;

          case 19:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, null, [[1, 14]]);
  }));
  return _install.apply(this, arguments);
}

function installBundle(_x2, _x3, _x4) {
  return _installBundle.apply(this, arguments);
}

function _installBundle() {
  _installBundle = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(bundle, repo, bundleDescriptor) {
    var _ref,
        isGlobal,
        type,
        dependencies,
        manifests,
        _args2 = arguments;

    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _ref = _args2.length > 3 && _args2[3] !== undefined ? _args2[3] : {}, isGlobal = _ref.isGlobal, type = _ref.type;
            dependencies = null;

            if (bundleDescriptor) {
              dependencies = parseInstallParam(bundleDescriptor);
              bundle.addDependencies(dependencies, type);
            }

            _context2.next = 5;
            return bundle.install(repo, isGlobal);

          case 5:
            manifests = _context2.sent;
            _context2.next = 8;
            return traverseDownload(manifests, isGlobal);

          case 8:
            if (dependencies && dependencies.length > 0) {
              bundle.updateDependencies(dependencies, type);
              bundle.updateBundleJson();
            }

          case 9:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));
  return _installBundle.apply(this, arguments);
}

function parseInstallParam(bundleDescriptor) {
  var list = bundleDescriptor.split(",");
  var dependencies = [];
  list.map(function (nameVersion) {
    var trimed = nameVersion.trim();
    var index = trimed.lastIndexOf("@");
    var version = (index > 0 ? trimed.substr(index + 1) : '') || "*"; // index === 0 is scoped name

    var name = index > 0 ? trimed.substr(0, index).trim() : trimed;

    if (_manifest.Manifest.Validator.name(name) !== true) {
      throw (0, _i18n.getI18nMessage)("install.validator", {
        name: name,
        validateMessage: _manifest.Manifest.Validator.name(name)
      });
    }

    if (!_semver["default"].validRange(version)) {
      throw (0, _i18n.getI18nMessage)("install.invalidVersionRange", {
        name: name
      });
    }

    dependencies.push(new _dependency.Dependency(name, version));
  });
  return dependencies;
}

function traverseDownload(manifests, isGlobal) {
  return new Promise(function (resolve, rejects) {
    var needDownload = [].concat(manifests).filter(function (manifest) {
      return !!manifest.url;
    });

    if (needDownload.length === 0) {
      resolve([]);
      return;
    }

    var threadEntryFile = _path["default"].join(__dirname, "..", "thread", "installEntry.js");

    var downloadCnt = 0;
    var allWorker = [];
    needDownload.forEach(function (manifest) {
      _i18n["default"].log("install.downloading", {
        name: manifest.name
      });

      var worker = new Worker(threadEntryFile, {
        workerData: {
          manifest: manifest.json(true),
          // 不可以直接传整个manifest进去，因为没法clone给子对象
          isGlobal: isGlobal
        }
      });
      worker.on("exit", function (errCode) {
        if (errCode === 0) {
          if (++downloadCnt === needDownload.length) {
            resolve(manifests);
            return;
          }
        } else {
          (0, _tools.closeAllThread)(allWorker);
          rejects((0, _i18n.getI18nMessage)("error.workerStop", {
            errCode: errCode
          }));
          return;
        }
      });
      worker.on("message", function (manifestPath) {
        manifests.forEach(function (m) {
          if (m.name === manifest.name) {
            m.path = manifestPath;
          }
        });
      });
      allWorker.push(worker);
    });
  });
}