"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.list = list;
exports.buildPrintData = buildPrintData;
exports.printDependencies = printDependencies;

var _path = _interopRequireDefault(require("path"));

var _os = _interopRequireDefault(require("os"));

var _dependencyGraph = require("../core/dependency-graph");

var _bundle = require("../core/bundle");

var _constant = require("../constant/constant");

var _repository = require("../core/repository");

var _config = require("../utils/config");

var _i18n = _interopRequireDefault(require("../i18n"));

var _log = _interopRequireDefault(require("../core/log"));

var _helpers = require("../utils/helpers");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function list() {
  return _list.apply(this, arguments);
}

function _list() {
  _list = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
    var cwd, bundle, repo, data;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.prev = 0;
            cwd = process.cwd();
            bundle = _bundle.Bundle.from(cwd);
            repo = new _repository.Repository(_path["default"].join(cwd, _constant.DEP_FOLDER_NAME), (0, _config.getGlobalRepo)());
            _context.next = 6;
            return buildPrintData(bundle, repo);

          case 6:
            data = _context.sent;
            printDependencies(data);
            _context.next = 15;
            break;

          case 10:
            _context.prev = 10;
            _context.t0 = _context["catch"](0);
            process.exitCode = -1;

            _i18n["default"].log("list.error", {
              error: (0, _helpers.getErrorMessage)(_context.t0)
            });

            _log["default"].debug(_context.t0);

          case 15:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, null, [[0, 10]]);
  }));
  return _list.apply(this, arguments);
}

function buildPrintData(_x, _x2) {
  return _buildPrintData.apply(this, arguments);
}

function _buildPrintData() {
  _buildPrintData = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(bundle, repo) {
    var manifestGraph, result, traverse;
    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            traverse = function _traverse(parentManifest, parentLevel) {
              result.push({
                name: parentManifest.name,
                version: parentManifest.version.version,
                level: parentLevel
              });
              var manifests = manifestGraph.dependencies(parentManifest);
              manifests.forEach(function (m) {
                traverse(m, parentLevel + 1);
              });
            };

            _context2.next = 3;
            return _dependencyGraph.DependencyGraph.build(bundle, repo);

          case 3:
            manifestGraph = _context2.sent;
            result = [];
            traverse(bundle.manifest, 0);
            return _context2.abrupt("return", result);

          case 7:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));
  return _buildPrintData.apply(this, arguments);
}

function printDependencies(data) {
  var message = "";
  data.forEach(function (d) {
    return message += "".concat(''.padEnd(d.level * 2, '| '), "+--").concat(d.name, "@").concat(d.version).concat(_os["default"].EOL);
  });

  _i18n["default"].log("common.message", {
    message: message
  });
}