"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.pack = pack;
exports.packBundle = packBundle;

var _bundle = require("../core/bundle");

var _tar = _interopRequireDefault(require("tar"));

var _fs = _interopRequireDefault(require("fs"));

var _path = _interopRequireDefault(require("path"));

var _helpers = require("../utils/helpers");

var _i18n = _interopRequireDefault(require("../i18n"));

var _log = _interopRequireDefault(require("../core/log"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _createForOfIteratorHelper(o, allowArrayLike) { var it; if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = o[Symbol.iterator](); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it["return"] != null) it["return"](); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function pack() {
  try {
    var bundle = _bundle.Bundle.from(process.cwd());

    packBundle(bundle);
  } catch (error) {
    process.exitCode = -1;

    _i18n["default"].log("pack.error", {
      error: (0, _helpers.getErrorMessage)(error)
    });

    _log["default"].debug(error);
  }
}

function packBundle(bundle) {
  var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      needSourceBin = _ref.needSourceBin,
      _ref$silent = _ref.silent,
      silent = _ref$silent === void 0 ? false : _ref$silent;

  var tgz = "".concat(bundle.id, ".tgz");

  if (_fs["default"].existsSync(tgz)) {
    (0, _helpers.runShellCmd)(function (shell) {
      return shell.rm('-rf', tgz);
    });
  }

  if (!needSourceBin) {
    var prePackCmd = bundle.scripts('pre_pack');

    if (prePackCmd) {
      (0, _helpers.runCmdWithSpawnSync)(function (spawnSync) {
        return spawnSync(prePackCmd, {
          shell: process.env.shellPath,
          env: bundle.vars.env,
          cwd: bundle.path,
          stdio: "inherit"
        });
      });
    }
  }

  _i18n["default"].log('pack.step1', {
    tgz: tgz,
    bundlePath: bundle.path
  }, {
    silent: silent
  });

  var base = _path["default"].join((0, _helpers.runShellCmd)(function (shell) {
    return shell.tempdir();
  }, false), "".concat(bundle.id, "-").concat(Date.now()));

  (0, _helpers.runShellCmd)(function (shell) {
    return shell.mkdir('-p', base);
  });
  var files = bundle.pack({
    needSourceBin: needSourceBin
  });

  if (!files) {
    (0, _helpers.runShellCmd)(function (shell) {
      return shell.cp('-R', _path["default"].resolve(bundle.path), base);
    });
  } else {
    var _loop = function _loop(dir) {
      _i18n["default"].log('pack.step2', {
        dir: dir
      }, {
        silent: silent
      });

      (0, _helpers.runShellCmd)(function (shell) {
        return shell.mkdir('-p', _path["default"].join(base, dir));
      });

      var _iterator = _createForOfIteratorHelper(files[dir]),
          _step;

      try {
        var _loop2 = function _loop2() {
          var file = _step.value;

          var fileRelativePath = _path["default"].relative(bundle.path, file);

          _i18n["default"].log("pack.step3", {
            fileRelativePath: fileRelativePath
          }, {
            silent: silent
          });

          (0, _helpers.runShellCmd)(function (shell) {
            return shell.cp('-R', file, _path["default"].join(base, dir));
          });
        };

        for (_iterator.s(); !(_step = _iterator.n()).done;) {
          _loop2();
        }
      } catch (err) {
        _iterator.e(err);
      } finally {
        _iterator.f();
      }
    };

    for (var dir in files) {
      _loop(dir);
    }
  }

  var dirname = _path["default"].parse(bundle.path).base;

  var temSourcePath = bundle.isPublishAs("code-segment") || bundle.isPublishAs("template") ? _path["default"].join(base, dirname) : base;

  var tarFiles = _fs["default"].readdirSync(temSourcePath);

  tarFiles = tarFiles.map(function (f) {
    return f.startsWith('@') ? "./".concat(f) : f;
  }); // tar-node

  _tar["default"].create({
    gzip: true,
    file: tgz,
    cwd: temSourcePath,
    sync: true
  }, tarFiles);

  _i18n["default"].log("pack.step4", {
    tgz: tgz
  }, {
    silent: silent
  });

  if (!needSourceBin) {
    var afterPackCmd = bundle.scripts('pack') || bundle.scripts('after_pack');

    if (afterPackCmd) {
      (0, _helpers.runCmdWithSpawnSync)(function (spawnSync) {
        return spawnSync(afterPackCmd, {
          shell: process.env.shellPath,
          env: bundle.vars.env,
          cwd: bundle.path,
          stdio: "inherit"
        });
      });
    }
  }
}