"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.template = exports.parseTemplate = void 0;

var _tar = _interopRequireDefault(require("tar"));

var _fs = _interopRequireDefault(require("fs"));

var _path = _interopRequireDefault(require("path"));

var _helpers = require("../utils/helpers");

var _remoteRepository = require("./remote-repository");

var _i18n = require("../i18n");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

var parseTemplate = /*#__PURE__*/function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(templatename, dirname) {
    var repo, trimed, index, templateVersion, templateParseName, manifest, templateDirPath;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.t0 = templatename;
            _context.next = _context.t0 === "simple" ? 3 : _context.t0 === "default" ? 3 : _context.t0 === "dist" ? 3 : 4;
            break;

          case 3:
            return _context.abrupt("return", template(templatename, dirname));

          case 4:
            repo = new _remoteRepository.RemoteRepository();
            trimed = templatename.trim();
            index = trimed.lastIndexOf("@");
            templateVersion = (index > 0 ? trimed.substr(index + 1) : '') || "*"; // index === 0 is scoped name

            templateParseName = index > 0 ? trimed.substr(0, index).trim() : trimed;
            _context.next = 11;
            return repo.manifest(templateParseName, templateVersion);

          case 11:
            manifest = _context.sent;

            if (manifest) {
              _context.next = 14;
              break;
            }

            throw (0, _i18n.getI18nMessage)("template.notFound", {
              name: templatename
            });

          case 14:
            if (!(manifest.publishAs !== "template")) {
              _context.next = 16;
              break;
            }

            throw (0, _i18n.getI18nMessage)("template.isNotTemplate", {
              name: templateParseName
            });

          case 16:
            templateDirPath = _path["default"].join(__dirname, "..", "asset", "template");
            _context.next = 19;
            return (0, _helpers.download)(manifest, templateDirPath);

          case 19:
            return _context.abrupt("return", template(manifest.id, dirname, true));

          case 20:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function parseTemplate(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

exports.parseTemplate = parseTemplate;

var template = function template(name) {
  var dirname = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : process.cwd();
  var isRemote = arguments.length > 2 ? arguments[2] : undefined;
  var manifestJson = {};

  var templatePath = _path["default"].join(__dirname, "..", "asset", "template", isRemote ? "" : "default", "".concat(name, ".tgz"));

  if (_fs["default"].existsSync(templatePath)) {
    _tar["default"].extract({
      file: templatePath,
      cwd: dirname,
      sync: true
    });
  }

  if (_fs["default"].existsSync(_path["default"].join(dirname, "bundle.json"))) {
    var bundle = _fs["default"].readFileSync(_path["default"].join(dirname, "bundle.json"));

    manifestJson = JSON.parse(bundle.toString());
    (0, _helpers.runShellCmd)(function (shell) {
      return shell.rm(_path["default"].join(dirname, "bundle.json"));
    });
  }

  return {
    templatePath: templatePath,
    manifestJson: manifestJson
  };
};

exports.template = template;