"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getI18nMessage = getI18nMessage;
exports["default"] = void 0;

var _en = _interopRequireDefault(require("./res/en"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var i18n = {
  log: function log(key, params) {
    var _ref = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {},
        _ref$silent = _ref.silent,
        silent = _ref$silent === void 0 ? false : _ref$silent;

    if (!silent) {
      var logMessage = parse(key, params);

      if (logMessage) {
        process.stdout.write("".concat(logMessage, "\n"));
      }
    }
  }
};

function parse(key, params) {
  var resource = getResource();
  var keys = key.split(".");
  var res = resource;

  while (keys.length > 0 && res) {
    var shift = keys.shift();
    res = res[shift] || null;
  }

  if (typeof res === "string" && params) {
    var message = parseMessage(res, params);
    return message;
  }

  return res || "";
}

function parseMessage(message, params) {
  var start = -1;
  var captures = [];

  for (var i = 0; i < message.length; i++) {
    var msg = message[i];
    var msgNext = message[i + 1];

    if (msg === "{" && msgNext && msgNext === "{") {
      start = i;
    }

    if (start >= 0 && msg === "}" && msgNext && msgNext === "}") {
      captures.push({
        start: start,
        end: i + 2,
        word: message.substr(start + 2, i - start - 2).trim()
      });
      start = -1;
    }
  }

  var result = message;

  for (var _i = captures.length - 1; _i >= 0; _i--) {
    var capture = captures[_i];

    if (capture.word && params[capture.word] !== undefined && capture.start >= 0 && capture.end >= 0) {
      result = result.substr(0, capture.start) + params[capture.word] + result.substr(capture.end);
    }
  }

  return result;
}

function getResource() {
  return _en["default"];
}

function getI18nMessage(key, params) {
  return parse(key, params);
}

var _default = i18n;
exports["default"] = _default;