"use strict";

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.shouldProxy = shouldProxy;
exports["default"] = exports.axios = void 0;

var _axios = _interopRequireDefault(require("axios"));

var _https = _interopRequireDefault(require("https"));

var _url = _interopRequireDefault(require("url"));

var _httpsProxyAgent = _interopRequireDefault(require("https-proxy-agent"));

var _i18n = _interopRequireWildcard(require("../i18n"));

var _config = require("./config");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var axios = _axios["default"].create({
  proxy: false
});

exports.axios = axios;
axios.interceptors.request.use(function (config) {
  _i18n["default"].log("request.requestUrl", {
    url: config.url
  });

  var proxySettings = getProxySettings(config.url);
  Object.keys(proxySettings).forEach(function (key) {
    config[key] = proxySettings[key];
  });

  if (config.method === "get" && config.params) {
    var keys = Object.keys(config.params);
    config.url += "?";
    keys.forEach(function (key) {
      config.url += "".concat(key, "=").concat(encodeURIComponent(config.params[key]), "&");
    });
    config.params = {};
  }

  return config;
});
axios.interceptors.response.use(function (res) {
  var data = res.data;

  if (data.code && data.code !== 200 && data.code !== 302) {
    throw data;
  }

  return res;
});

function getProxySettings(visitingUrl) {
  var visitingUrlParsed = _url["default"].parse(visitingUrl);

  var noProxy = process.env["no_proxy"] || process.env["NO_PROXY"];

  if (!shouldProxy(noProxy, visitingUrlParsed.hostname)) {
    return {
      httpsAgent: new _https["default"].Agent({
        rejectUnauthorized: (0, _config.isStrictSsl)()
      })
    };
  } else {
    if (!(0, _config.isStrictSsl)()) {
      process.removeAllListeners('warning');
      process.env['NODE_TLS_REJECT_UNAUTHORIZED'] = 0;
    }

    var httpsAgent;
    var httpAgent;
    var httpsProxy = process.env["https_proxy"] || process.env["HTTPS_PROXY"];

    if (httpsProxy) {
      var httpsProxyParsed = parseProxyUrl(httpsProxy);

      var opts = _url["default"].parse(httpsProxyParsed.proxypath);

      if (httpsProxyParsed.auth) {
        opts.auth = httpsProxyParsed.auth;
      }

      httpsAgent = new _httpsProxyAgent["default"](opts);
    }

    var httpProxy = process.env["http_proxy"] || process.env["HTTP_PROXY"];

    if (httpProxy) {
      var httpProxyParsed = parseProxyUrl(httpProxy);

      var _opts = _url["default"].parse(httpProxyParsed.proxypath);

      _opts.protocol = 'http';

      if (httpProxyParsed.auth) {
        _opts.auth = httpProxyParsed.auth;
      }

      httpAgent = new _httpsProxyAgent["default"](_opts);
    }

    return {
      httpsAgent: httpsAgent,
      httpAgent: httpAgent
    };
  }
}

function parseProxyUrl(proxyUrl) {
  var withAuthParsed = /(https:\/\/|http:\/\/)([\s\S]+)@([\s\S]+)/g.exec(proxyUrl);
  var proxypath;
  var auth;

  if (withAuthParsed) {
    if (withAuthParsed.length !== 4) {
      throw (0, _i18n.getI18nMessage)("request.proxyFormat");
    } else {
      auth = withAuthParsed[2];
      proxypath = "".concat(withAuthParsed[1]).concat(withAuthParsed[3]);
    }
  } else {
    proxypath = proxyUrl;
  }

  return {
    proxypath: proxypath,
    auth: auth
  };
}

function shouldProxy(noProxy, hostname) {
  if (!noProxy) {
    return true;
  }

  var noProxyArr = noProxy.split(",").map(function (s) {
    return s.trim();
  });
  var should = !noProxyArr.some(function (proxyElement) {
    if (!proxyElement) {
      return false;
    }

    if (proxyElement === "*") {
      return true;
    }

    if (proxyElement[0] === "." && hostname.substr(hostname.length - proxyElement.length) === proxyElement) {
      return true;
    }

    if (proxyElement[0] === "*" && hostname.substr(hostname.length - proxyElement.length + 1) === proxyElement.substr(1)) {
      return true;
    }

    return hostname === proxyElement;
  });
  return should;
}

var _default = axios;
exports["default"] = _default;