"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isStrictSsl = isStrictSsl;
exports.getRootBundlePath = exports.getGlobalRepo = exports.getConfig = exports.initConfig = void 0;

var _fs = _interopRequireDefault(require("fs"));

var _path = _interopRequireDefault(require("path"));

var _rcParser2 = _interopRequireDefault(require("../core/rc-parser"));

var _helpers = require("./helpers");

var _constant = require("../constant/constant");

var _config = _interopRequireDefault(require("../config.json"));

var _i18n = _interopRequireDefault(require("../i18n"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var defaultConfig = {
  registry: _config["default"].registry,
  login: _config["default"].login,
  globalRepo: _constant.DEFAULT_GLOBAL_DIR,
  strictSsl: _config["default"].strictSsl
};

if ((0, _helpers.isWindows)()) {
  defaultConfig.shellPath = process.env.COMSPEC;
} else {
  defaultConfig.shellPath = _config["default"].unixShell;
}

var initConfig = function initConfig() {
  if (!_fs["default"].existsSync(_constant.DEFAULT_CONFIG_DIR)) {
    (0, _helpers.runShellCmd)(function (shell) {
      return shell.mkdir('-p', _constant.DEFAULT_CONFIG_DIR);
    });
  }

  var configPath = _constant.DEFAULT_CONFIG_FILE;
  var config = null; // init hpmrc

  if (!_fs["default"].existsSync(configPath)) {
    var rcParser = _rcParser2["default"].saveJsonToFile(defaultConfig, configPath);

    config = rcParser.toJson();
  } else {
    var _rcParser = _rcParser2["default"].fromPath(configPath);

    var json = _rcParser.toJson();

    config = _objectSpread(_objectSpread({}, defaultConfig), json);

    if (config.shellPath && !_fs["default"].existsSync(config.shellPath)) {
      _i18n["default"].log("config.shellPathNotFound", {
        shellPath: config.shellPath
      });

      _i18n["default"].log("common.modifyRc", {
        filePath: _constant.DEFAULT_CONFIG_FILE
      });

      process.exit(-1);
    }
  } // init global bundle json


  if (!_fs["default"].existsSync(config.globalRepo)) {
    (0, _helpers.runShellCmd)(function (shell) {
      return shell.mkdir('-p', config.globalRepo);
    });

    _fs["default"].writeFileSync(_path["default"].join(config.globalRepo, 'bundle.json'), JSON.stringify({
      name: 'global',
      version: '0.0.1'
    }, null, 4));
  } // write config to process.env


  for (var key in config) {
    if (config[key]) {
      if (key.toLowerCase() === 'path') {
        var pathArr = config[key].split(_path["default"].delimiter).concat(process.env.path.split(_path["default"].delimiter));
        process.env.path = pathArr.join(_path["default"].delimiter);
      } else {
        process.env[key] = config[key];
      }
    }
  }
};

exports.initConfig = initConfig;

var getConfig = function getConfig() {
  var configPath = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : _constant.DEFAULT_CONFIG_FILE;

  if (_fs["default"].existsSync(configPath)) {
    var rcParser = _rcParser2["default"].fromPath(configPath);

    var config = rcParser.toJson();
    return config;
  }

  return defaultConfig;
};

exports.getConfig = getConfig;

var getGlobalRepo = function getGlobalRepo() {
  return _path["default"].join(process.env.globalRepo, _constant.DEP_FOLDER_NAME);
};

exports.getGlobalRepo = getGlobalRepo;

var getRootBundlePath = function getRootBundlePath(isGlobal) {
  return isGlobal ? process.env.globalRepo : process.cwd();
};

exports.getRootBundlePath = getRootBundlePath;

function isStrictSsl() {
  var strictSsl = process.env["strictSsl"];

  if (typeof strictSsl === 'string') {
    return strictSsl.trim().toLowerCase() === 'true';
  }

  return false;
}